-- $Id$
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Quest = require 'engine.Quest'

store_util.set_prices(npc.store.owner, player, {
  id = {1400, 1300, 1200},
})

newChat {
  id = 'welcome',
  text = [[#LIGHT_GREEN#*A rather ostentatiously-dressed High-Elf librarian dusts off his jacket and straightens his spectacles as you enter.*#LAST#
Ah, welcome, noble adventurer, to the kingdom's renowned library.  Our stores of knowledge are vast, and our skilled staff can surely find detailed information about whatever magical items you may have found in your travels.  Or for your more plebeian identification needs, our gift shop sells Staffs of Identify.
#LIGHT_GREEN#*You hear a muffled thump and some ominous scrabbling noises from below the floor.  The librarian grins weakly.*#LAST#  Eh-heh.  Minor construction.  Nothing to be concerned about.]],
  answers = {
    {
      'Can you research this item for me?' .. store_util.cost_sfx('id'),
      cond = store_util.ok_full_id_item,
      action = store_util.full_id_item,
    },
    {
      "Ooh, Staffs of Identify, I'm fresh out of those.",
      action = function(npc, player)
	npc.store:loadup(game.level, game.zone)
	npc.store:interact(player)
      end,
    },
    {
      [[That... didn't sound like any construction I've ever heard.]],
      jump = 'quest1',
    },
    { 'Oh, sorry, I was looking for the weapon store. [leave]' },
  },
}

newChat {
  id = 'quest1',
  text = [[#LIGHT_GREEN#*The librarian glances around to see if anyone's listening, then sighs resignedly.*#LAST#
All right, we're not supposed to tell customers about this, but we're having a bit of a problem in the deep stacks.  It seems one of our novice researchers fancied himself a budding necromancer, and was using the relative seclusion of the stacks to quietly amass his "undead army of vengeance" or some such rot.  And #{italic}#naturally#{normal}# the lackwit managed to get himself offed by his own creations, leaving them prowling about leaderless and blocking access to our most important manuscript collections.  The last team of librarians we sent down to investigate barely got out with their lives.]],
  answers = {
    {
      'Ooh, undead?  That you need killed?  I happen to be pretty good at that sort of thing.',
      jump = 'quest2',
    },
    { 'Wow.  Sucks to be you guys. [leave]' },
  },
}

newChat {
  id = 'quest2',
  text = [[Oh my, could I possibly prevail upon you to clean up this mess?  I can make it worth your while; once our scribes get access to the stacks, they can prepare a custom spellbook for you, with three spells of your choosing.  I'll tell security to allow you in at the side entrance to the stacks.]],
  answers = {
    {
      'Leave it to me.',
      action = function(npc, player)
	player:grantQuest('minas-anor-library')
	util.place_quest_entrance('quest', 'library', 'ENTER_MINAS_ANOR_LIBRARY_QUEST', true)
      end,
    },
  },
}

newChat {
  id = 'welcome2',
  text = [[Ah, welcome, noble adventurer.  #LIGHT_GREEN#*The librarian looks around and leans in close.*#LAST#  Any word on the deep stacks?]],
  answers = {
    { "I'm working on it. [leave]" },
    {
      "I'm working on it.  In the meantime, can you research this item for me?" .. store_util.cost_sfx('id'),
      cond = store_util.ok_full_id_item,
      action = store_util.full_id_item,
    },
    {
      "I'm working on it.  In the meantime, can I buy some more Staffs of Identify?",
      action = function(npc, player)
	npc.store:loadup(game.level, game.zone)
	npc.store:interact(player)
      end,
    },
  },
}

newChat {
  id = 'welcome3',
  text = [[Ah, noble adventurer!  Dare I hope all those loud noises from the stacks portend good news?]],
  answers = {
    {
      'Yes, I have cleared out all the undead.',
      cond = function(npc, player)
	return player:isQuestStatus('minas-anor-library', Quest.COMPLETED)
      end,
      jump = 'quest3',
    },
    {
      "No, I'm afraid I barely got out with my life.",
      cond = function(npc, player)
	return player:isQuestStatus('minas-anor-library', Quest.FAILED)
      end,
      jump = 'quest4',
    },
  },
}

newChat {
  id = 'quest3',
  text = [[Splendid!  Come, I shall fetch the scribes and we shall prepare your spellbook right away.]],
  answers = {
    {
      'Cool.',
      action = function(npc, player)
	local Spellbook = require 'mod.dialogs.LibraryQuestSpellbook'
	game:registerDialog(Spellbook.new(player))
      end,
    }
  },
}

newChat {
  id = 'quest4',
  text = [[Ah.  Oh well, at least you tried, which is better than most can claim.  Can I be of any further assistance?]],
  answers = {
    {
      'Can you research this item for me?' .. store_util.cost_sfx('id'),
      cond = store_util.ok_full_id_item,
      action = store_util.full_id_item,
    },
    {
      'I need some more Staffs of Identify.',
      action = function(npc, player)
	npc.store:loadup(game.level, game.zone)
	npc.store:interact(player)
      end,
    },
    { "Nah, I'm good.  Thanks anyway. [leave]" },
  },
}

newChat {
  id = 'welcome4',
  text = [[Ah, most esteemed adventurer!  How may I be of assistance?]],
  answers = {
    {
      'Can you research this item for me?' .. store_util.cost_sfx('id'),
      cond = store_util.ok_full_id_item,
      action = store_util.full_id_item,
    },
    {
      'I need some more Staffs of Identify.',
      action = function(npc, player)
	npc.store:loadup(game.level, game.zone)
	npc.store:interact(player)
      end,
    },
    { 'Oh, sorry, I was looking for the weapon store. [leave]' },
  },
}

local q = player:hasQuest('minas-anor-library')
if not q then
  return 'welcome'
elseif q:isEnded() and not q:isFailed() then
  return 'welcome4'
elseif q:isCompleted() or q:isFailed() then
  return 'welcome3'
else
  return 'welcome2'
end
