-- $Id: quest-lost-sword-finish.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local lev = game.zone:level_adjust_level(game.level, 'actor')
local state = game.state.sword_quests[lev]

local function pick_skills()
  local Skills = require 'mod.class.interface.PlayerSkills'
  local cands = {}
  local ret = {}
  local excl = {}

  for tag, def in pairs(Skills.skills_def) do
    if def.random_gain then cands[#cands+1] = tag end
    -- We'll be nicer than T2 source and exclude skills that conflict with
    -- the player's skills.
    if def.exclude then
      -- If player has this skill, mark excluded skills to exclude.
      if game.player:getSkillLevel(tag, 50, true) > 0 then
	for _, ex in ipairs(def.exclude) do excl[ex] = true end
      end
      -- If player has any of the excluded skills, mark this skill to
      -- exclude
      for _, ex in ipairs(def.exclude) do
	if game.player:getSkillLevel(ex, 50, true) > 0 then
	  excl[tag] = true
	  break
	end
      end
    end
  end

  while #ret < 4 do
    local i = rng.range(1, #cands)
    local tag, def = cands[i], Skills.skills_def[cands[i]]

    if rng.percent(def.random_gain) and not excl[tag] then
      local sk = game.player.skills[tag]
      if sk.mod == 0 then
	ret[#ret+1] = { id=tag, mod=300, value=1000 }
      elseif sk.mod < 300 then
	ret[#ret+1] = { id=tag, mod=300-sk.mod, value=1000 }
      elseif sk.mod < 500 then
	ret[#ret+1] = { id=tag, mod=math.max(100, 500-sk.mod), value=sk.mod }
      else
	ret[#ret+1] = { id=tag, mod=0, value=sk.mod*3 }
      end
      ret[#ret].name = def.name
      table.remove(cands, i)
    end
  end

  return ret
end

local choices = pick_skills()
local choices_desc = table.concatNice(table.mapv(function(v) return v.name end, choices), ', ', ' and ')

local reward_answers = {}
for i, choice in ipairs(choices) do
  local t = { jump='finish' }
  local desc
  if choice.mod > 0 then
    desc = ('%+.2f mod, %+.2f skill'):format(choice.mod/1000, choice.value/1000)
  else
    desc = ('%+.2f skill'):format(choice.value/1000)
  end
  t[1] = choice.name .. ', please (#LIGHT_BLUE#' .. desc .. ').'
  t.action = function(npc, player, dlg)
    local lev = game.zone:level_adjust_level(game.level, 'actor')
    local state = game.state.sword_quests[lev]
    local sk = game.player.skills[choice.id]
    sk.mod = sk.mod + choice.mod
    sk.value = sk.value + choice.value
    -- Remember what reward we chose.
    local q = player:hasQuest(state.q_id)
    if q then
      q.q_reward_skill = choice.name
      q.q_reward_mod = choice.mod
      q.q_reward_value = choice.value
    end
    -- Done the quest.
    player:setQuestStatus(state.q_id, engine.Quest.DONE)
    -- Sneaky Hack(TM):  Manually munge the next chat object's text to
    -- match the reward.
    local text = [[#LIGHT_GREEN#*The adventurer touches your forehead, and ]]
    if choice.mod > 0 then
      text = text .. 'you gain knowledge of the ' .. choice.name .. ' skill.'
    else
      text = text .. 'your knowledge of the ' .. choice.name .. ' skill increases.'
    end
    text = text .. [[.*#LAST#  And done.  Good luck on your journeys.]]
    local chat = dlg.chat:get('finish')
    chat.text = text
  end
  table.insert(reward_answers, t)
end


local lastguy = state.n == 1 and state.name or 'the last '..state.name

newChat {
  id = 'begin',
  text = [[#LIGHT_GREEN#*As ]]..lastguy..[[ falls, the adventurer reemerges from the shadows.*#LAST#
Ah!  My sword, at last!  It figures the very last one would have it, eh?]],
  answers = {
    { [[#{italic}#*shrug*#{normal}# Story of my life.]], jump='reward' },
  },
}

newChat {
  id = 'reward',
  text = [[Let me do something to thank you.  As it happens, I'm something of a trainer; perhaps I can teach you something.  I'm a bit of a polyglot, actually; I specialize in ]] .. choices_desc .. '.',
  answers = reward_answers,
}

newChat {
  id = 'finish',
  text = [[#RED#ERROR:  If you are seeing this text, something has gone wrong.#LAST#]],
  answers = {
    { 'Thank you.' },
  },
}

return 'begin'
