-- $Id: turgon.lua 3294 2018-03-14 04:51:28Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Quest = require 'engine.Quest'

newChat {
  id = 'welcome',
  text = [[Ah, @playername@.]],
  answers = {
    {
      'I have cleared out the dragons, Your Highness.',
      cond = function(npc, player)
	return player:isQuestStatus('gondolin-house', Quest.COMPLETED)
      end,
      jump = 'house-done',
    },
    {
      'I have slain Eöl, Your Highness.',
      cond = function(npc, player)
	return player:isQuestStatus('eol-the-dark-elf', Quest.COMPLETED)
      end,
      jump = 'eol-done',
    },
    {
      "I'm afraid Eöl escaped, Your Highness.",
      cond = function(npc, player)
	return player:isQuestStatus('eol-the-dark-elf', Quest.FAILED)
      end,
      jump = 'eol-fail',
    },
    {
      'Your armies have retreated back into Gondolin, Your Highness.',
      cond = function(npc, player)
	return player:isQuestStatus('nirnaeth-arnoediad', Quest.COMPLETED)
      end,
      action = function(npc, player)
	-- Jump to a different chat depending on how many trolls we killed.
	if player:isQuestStatus('nirnaeth-arnoediad', Quest.COMPLETED, 'killed-most') then
	  return 'nirnaeth-done-extra'
	end
	return 'nirnaeth-done'
      end,
    },
    {
      'Maeglin is dead, Your Highness',
      cond = function(npc, player)
	return player:isQuestStatus('invasion-of-gondolin', Quest.COMPLETED)
      end,
      jump = 'invasion-done',
    },
    { 'Pardon me, Your Highness. [leave]', },
  },
}

newChat {
  id = 'give-house',
  text = [[#LIGHT_GREEN#*You find King Turgon here in his throne room.*#LAST#
@playername@.  Know that I do not lightly allow outsiders into the hidden city of Gondolin.  You are here because tales of your deeds elsewhere lead me to believe that you can be of use for a task I need done.]],
  answers = {
    { 'That seems to be a recurring theme...', jump='house-quest' },
  },
}

newChat {
  id = 'house-quest',
  text = [[Our high altitude in the Encircling Mountains protects us, but it also seems to draw dragons.  A brood of them have apparently occupied an abandoned storehouse in the northwest part of the city, and from there they have made a nuisance of themselves.  Clear them out and I will declare you welcome in Gondolin.]],
  answers = {
    {
      "Doesn't sound too hard.  I'm in.",
      action = function(npc, player)
	player:grantQuest('gondolin-house')
	util.place_quest_entrance('quest', 'house', 'ENTER_GONDOLIN_HOUSE_QUEST', true)
      end,
    },
    { '#{italic}#*groan*#{normal}# Man, I am just dragoned out right now.  Lemme get back to you on that. [leave]' },
  },
}

newChat {
  id = 'house-done',
  text = [[So I am told.  Perhaps you are worth allowing into Gondolin after all.  Check in with me from time to time; I will no doubt have more tasks for you to prove your value to the city.  Mm, you will need lodging, though, I suppose... ah, but there is an empty storehouse now, isn't there?  You may use that; I will make the arrangements.]],
  answers = {
    {
      'Umm... thank you, Your Highness.',
      action = function(npc, player)
	player:setQuestStatus('gondolin-house', Quest.DONE)
	util.place_home('quest', 'house', 'PERMA_WALL', 'GONDOLIN_HOME')
      end,
    },
  },
}

newChat {
  id = 'give-eol',
  text = [[Ah, @playername@, just the catspaw I was thinking of.]],
  answers = {
    { 'Oh, this cannot possibly end well...', jump='eol-quest' },
  },
}

newChat {
  id = 'eol-quest',
  text = [[My scouts have spotted Eöl the Dark Elf skulking around the eastern mountains trying to find our hidden city.  He is no doubt seeking his wife, my sister Aredhel.  Unlike you, however, he is of no use to me, and I have no intention of letting him enter Gondolin.  You will take the pass through the eastern mountains and apprehend Eöl.  If he will agree to leave, let him; if he persists, you have my permission to kill him if you must.  Be wary, though — the scouts say he has been laying traps against bandits.]],
  answers = {
    {
      "#{italic}#(shrug)#{normal}# It's a dirty job, and it sounds like I've got to do it.",
      action = function(npc, player)
	player:grantQuest('eol-the-dark-elf')
	util.place_quest_entrance('quest', 'eol', 'ENTER_GONDOLIN_EOL_QUEST', true)
      end,
    },
    { 'Yyyeah, I try to avoid getting involved in family squabbles. [leave]' },
  },
}

newChat {
  id = 'eol-done',
  text = [[So my scouts tell me.  I didn't think the sentimental fool would listen to reason.  But it had to be done, I suppose.  You've once again justified my decision to allow you access to Gondolin.  In fact, here is something our royal crafters just finished; take it as a reward.  I'm sure it will serve you better than me.]],
  answers = {
    {
      'Thank you, Your Highness.',
      action = function(npc, player)
	-- The quest's on_status_change() will give us the Dwarven Lantern
	-- of the Magi.
	player:setQuestStatus('eol-the-dark-elf', Quest.DONE)
      end,
    },
  },
}

newChat {
  id = 'eol-failed',
  text = [[Fortunately, you are mistaken on that matter.  According to my scouts, you weakened him enough that they were able to kill him after you left.  Damn sentimental fool...  So, not quite the win I was hoping for from you, but you did provide important help, so I will continue to allow you access to Gondolin.]],
  answers = {
    { 'Umm... thank you, Your Highness. [leave]' }
  },
}

newChat {
  id = 'give-nirnaeth',
  text = [[Ah, @playername@, my favorite catspaw.  Tell me, how are you against large armies?]],
  answers = {
    { "There's no possible answer I can give to that question that won't get me in trouble, is there?", jump='nirnaeth-quest' },
  }
}

newChat {
  id = 'nirnaeth-quest',
  text = [[Maedhros' assault on Thangorodrim has turned into a rout, as I'd feared.  Morgoth's treachery has driven our armies back nigh to the very walls of the city, and their retreat is cut off.  You will go forth from the city gates and clear a path for them to retreat into the city.  My word, I seem to have horrified you.  No, I don't expect you to single-handedly destroy the entire troll army; it will suffice to drive a path through them.]],
  answers = {
    {
      "I figured it'd be something like that.  Okay, I'm on it.",
      action = function(npc, player)
	player:grantQuest('nirnaeth-arnoediad')
	util.place_quest_entrance('quest', 'nirnaeth', 'ENTER_GONDOLIN_NIRNAETH_QUEST', true)
      end,
    },
    { "With the greatest of respect, Your Highness, you are out of your frickin' gourd. [leave]" },
  },
}

newChat {
  id = 'nirnaeth-done',
  text = [[Yes, my generals have already reported to me.  They tell me they would not have made it without your help.  As a token of their gratitude, I will grant you access to the royal jewelry shop.]],
  answers = {
    {
      'Thank you, Your Highness.',
      action = function(npc, player)
	-- The quest's on_status_change() will give us the corresponding
	-- achievement.
	player:setQuestStatus('nirnaeth-arnoediad', Quest.DONE)
	util.place_home('quest', 'jewelry', 'PERMA_WALL', 'RARE_JEWELRY_SHOP')
      end,
    },
  },
}

newChat {
  id = 'nirnaeth-done-extra',
  text = [[Yes, my generals have already told me of your remarkable assault against the trolls.  I must apologize for my harsh words earlier, @playername@; you have proven your valor and then some.  To think that one @playerdescriptor.race@ could put such a dent in Morgoth's armies... well, he will certainly think twice before attacking us again.  Here, I have released some funds from the royal treasury, so that you can fix up that old storehouse into some proper lodgings.  Oh, and the generals have asked me to give you another boon, as a token of their gratitude; I think I will grant you access to the royal jewelry shop.]],
  answers = {
    {
      'Thank you, Your Highness.',
      action = function(npc, player)
	-- The quest's on_status_change() will give us the corresponding
	-- achievement and the 200Kgp.
	player:setQuestStatus('nirnaeth-arnoediad', Quest.DONE)
	util.place_home('quest', 'jewelry', 'PERMA_WALL', 'RARE_JEWELRY_SHOP')
      end,
    },
  },
}

newChat {
  id = 'last-alliance',
  text = [[#LIGHT_GREEN#*Turgon is in his throne room, conferring with several Thunderlords.  They all turn to you as you enter.*#LAST#
Ah, Aragorn's courier, good.  Here is the information his Thunderlords will need to complete the void jumpgate.  Take it back to him quickly.  Be careful, though; my spies tell me the rebel Thunderlord Trone has gotten wind of your mission.  His loyalists will likely try to ambush you on the way back to Minas Anor.]],
  answers = {
    {
      'I will be careful, Your Highness.',
      action = function(npc, player)
	player:setQuestStatus('last-alliance', Quest.COMPLETED, 'gondolin')
      end,
    }
  },
}

newChat {
  id = 'last-alliance-alt',
  text = [[#LIGHT_GREEN#*Turgon is in his throne room, conferring with several Thunderlords.  They all turn to you as you enter.*#LAST#
Ah, @playername@.  So you are Aragorn's courier, then?  That works out well.  Here is the information his Thunderlords will need to complete the void jumpgate.  Take it back to him quickly.  Be careful, though; my spies tell me the rebel Thunderlord Trone has gotten wind of your mission.  His loyalists will likely try to ambush you on the way back to Minas Anor.]],
  answers = {
    {
      'I will be careful, Your Highness.',
      action = function(npc, player)
	player:setQuestStatus('last-alliance', Quest.COMPLETED, 'gondolin')
      end,
    }
  },
}

newChat {
  id = 'give-invasion',
  text = [[#LIGHT_GREEN#Turgon dismisses his advisors as you and Liron enter.*#LAST#
Ah, good, Liron found you at last.  We have a situation.  My accursed nephew Maeglin has betrayed us.  I #{italic}#knew#{normal}# I should never have entrusted him with knowledge of the secret routes into Gondolin.  Even now he is leading an army of Dragons and Balrogs through the Echoriath to assault us.]],
  answers = {
    { "That's not good.", jump='invasion-quest' },
  },
}

newChat {
  id = 'invasion-quest',
  text = [[We still have a chance, though.  Maeglin may be a traitor, but he is no fool.  He knows that if he tells Morgoth's forces how to use the secret entrances, he will no longer be of use to them and they will kill him.  That is why he leads the army himself.  If you can kill Maeglin before he reaches the secret entrance, his army will not be able to find or use it.]],
  answers = {
    {
      "I'm on it.",
      action = function(npc, player)
	player:setQuestStatus('invasion-of-gondolin', Quest.COMPLETED, 'turgon')
	util.place_quest_entrance('quest', 'invasion', 'ENTER_GONDOLIN_INVASION_QUEST', true)
      end,
    },
  },
}

newChat {
  id = 'invasion-done',
  text = [[That would certainly explain the behavior of Morgoth's army; they are making quite the mess in the Echoriath, trying to find the entrance to Gondolin, and becoming quite enraged by their failure.  They will be no threat now.  You have done us a great service, @playername@.  I have summoned my most powerful mages; they are going to cast a mighty spell on you, increasing your life.]],
  answers = {
    {
      'Thank you, Your Highness.',
      action = function(npc, player)
	-- The quest's on_status_change() will give us the HP bonus.
	player:setQuestStatus('invasion-of-gondolin', Quest.DONE)
      end,
    },
  },
}


local q_complete = {
  'gondolin-house', 'eol-the-dark-elf', 'nirnaeth-arnoediad',
  'invasion-of-gondolin',
}
local q_failed = { 'eol-the-dark-elf' }
local q_next = {
  { need='gondolin-house', jump='give-house' },
  { done='gondolin-house', need='eol-the-dark-elf', jump='give-eol' },
  { done='eol-the-dark-elf', need='nirnaeth-arnoediad', jump='give-nirnaeth' },
}

if player:hasQuest('last-alliance') and not player:isQuestStatus('last-alliance', Quest.COMPLETED, 'gondolin') then
  -- Slightly different chats, depending on whether we've met Turgon yet. ;)
  return player:hasQuest('gondolin-house') and 'last-alliance-alt' or 'last-alliance'
end
if player:hasQuest('invasion-of-gondolin') and not player:isQuestStatus('invasion-of-gondolin', Quest.COMPLETED, 'turgon') then
  return 'give-invasion'
end

for _, tag in ipairs(q_complete) do
  if player:isQuestStatus(tag, Quest.COMPLETED) then return 'welcome' end
end
for _, tag in ipairs(q_failed) do
  if player:isQuestStatus(tag, Quest.FAILED) then return 'welcome' end
end
for _, grp in ipairs(q_next) do
  if (not grp.done or player:isQuestStatus(grp.done, Quest.DONE)) and not player:hasQuest(grp.need) then
    return grp.jump
  end
end
return 'welcome'
