-- $Id: necromancer.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

add_hooks = function()
  class:bindHook('Game:enterLevel', function(self, data)
    local zone_id = data.zone.short_name
    local last = data.lev == data.zone.max_level
    local q = game.player:hasQuest('necromancer')
    if q and q.zones[zone_id] and last then
      -- Finished the zone; update the quest.
      game.player:setQuestStatus('necromancer', engine.Quest.COMPLETED, zone_id)
    end
  end)
end

zones = {
  'barrow-downs', 'mirkwood', 'mordor',
  ['barrow-downs'] = {
    [[The Barrow-Downs, just outside of Bree #YELLOW#(beginning at dungeon level 1)#LAST#]],
    [[#LIGHT_GREEN#You have cleared the Barrow-Downs.#LAST#]],
  },
  mirkwood = {
    [[The forests of Mirkwood, beyond the Misty Mountains to the east and near the Elven town of Lothlórien #YELLOW#(beginning at dungeon level 11)#LAST#]],
    [[#LIGHT_GREEN#You have cleared Mirkwood.#LAST#]],
  },
  mordor = {
    [[Sauron's abandoned fortress of Mordor, near the city of Minas Anor to the south #YELLOW#(beginning at dungeon level 34)#LAST#]],
    [[#LIGHT_GREEN#You have cleared Mordor.#LAST#]],
  },
}

id = 'necromancer'
name = 'Dol Guldur'
danger_level = 70	-- cf. src/tables.c line 3647
desc = function(self, who)
  if self:isEnded() then
    return [[You have driven the Necromancer out of the Tower of Dol Guldur, and notified the Elven Council that he was in fact Sauron the Sorcerer in disguise.]]
  elseif self:isCompleted() then
    return [[You have slain the Necromancer of Dol Guldur and discovered that he was indeed Sauron in disguise.  You should go to Lothlórien and tell Galadriel of this.]]
  else
    local text = [[To the south of Mirkwood lies the Tower of Dol Guldur, wherein resides the Necromancer of Dol Guldur, whose twisted monsters infest Mirkwood.  Rumor has it the Elven Council fear that the Necromancer is actually Sauron the Sorcerer in disguise; you figure someone should find out, and it might as well be you.]]
    -- I figure we can say the player isn't ready yet until they've been to
    -- level 50 of some dungeon.
    local any50 = false
    for zone_id, lev in pairs(game.state.zone_recall_depth) do
      local info = game.state.zone_info[zone_id]
      if info then
	any50 = any50 or info.level_range[1] + lev - 1 >= 50
      end
    end
    if not any50 then
      text = text .. [[  You're not nearly strong enough for that yet, though.  Local adventurers' lore suggests some places where you could hone your skills to prepare:]]
    end
    text = text .. '\n'
    for _, zone_id in ipairs(self.zones) do
      local idx = self:isStatus(engine.Quest.COMPLETED, zone_id) and 2 or 1
      text = text .. '\n  · ' .. self.zones[zone_id][idx]
    end
    if not any50 then
      text = text .. [[


You needn't do all of these, of course, if you're particularly brave (or if you find somewhere else to level up).]]
    end
    return text
  end
end

on_grant = function(self, who)
  -- Set the Thrain quest level, unadjusted by Zone:level_adjust_level().
  -- Vaguely Annoying Hack(TM):  We'd like to be able to pull max_level out
  -- of Dol Guldur's zone definition (14 as of this writing), but that's
  -- too much work, so we effectively hard-code it here.
  self.thrain_level = rng.range(2, 13) -- All but first and last level.
end

callbacks = {
  on_kill = function(who, q, target, rsrc)
    if target.define_as == 'THE_NECROMANCER_OF_DOL_GULDUR' then
      local Dialog = require 'engine.ui.Dialog'
      local txt = [[As the Necromancer falls, his body dissipates into nothing and you sense his spirit escape his body and flee.  With a shudder, you recognize the departing spirit as none other than Sauron.  It would seem the Elven Council were right in their suspicions.  You should return to Lothlórien and tell Galadriel of this.]]
      local function cb()
	game.player:setQuestStatus('necromancer', engine.Quest.COMPLETED)
      end
      Dialog:simpleLongPopup(target:getName(), txt, 600, cb)
    end
  end,
}
