-- $Id: zone.lua 4279 2021-09-18 23:39:31Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2021 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Zone file for Mordor.

return {
  name = 'Mordor',
  display_abbr = 'Mdr',
  level_scheme = 'fixed',
  max_level = 33,
  level_range = { 34, 66 },
  width = 198,
  height = 66,
  principal = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { weighted = { DIRT={67,0}, ASH={33,100} } },
      fill = { weighted = { MOUNTAIN_CHAIN={50,0}, WALL={50,100} } },
      outer_wall = 'WALL',
      inner_wall = 'MOUNTAIN_CHAIN',
      fill_step = 2,
      up = 'UP',
      down = 'DOWN',
      flags = {
	LAVA_RIVER = true,
	CAVERN = true,
	NO_STREAMERS = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 14,
      respawn_in = 160,
      filters = {
	{ prob=100, mode='any' },
      },
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=20, combat=20, magic=20, tools=20, junk=20 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_WILDERNESS',
	  -- up_lev = 1
	},
      },
    },
    [32] = {
      generator = {
	map = {
	  down_branch = 'DOWN_TO_MOUNT_DOOM',
	},
      },
    },
  },

  entry_point = function(self, lev, from_lev, from_zone)
    if not from_zone then
      return nil
    elseif from_zone.name == 'mount-doom' then
      -- Returning from a down branch; enter on a down-branch stair.
      return { type='branch', subtype='down' }
    elseif util.zone_is(from_zone, 'worldmap') or util.zone_is(from_zone, 'wilderness') then
      -- Coming from the world map (or Bree -> Barrow-Downs); start on an
      -- up stair.
      return { use='default_up' }
    else
      return nil
    end
  end,
}
