-- $Id: zone.lua 4279 2021-09-18 23:39:31Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2021 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Zone file for Mount Doom.

local DamageType = require('engine.DamageType')

return {
  name = 'Mount Doom',
  display_abbr = 'MDm',
  level_scheme = 'fixed',
  max_level = 15,
  level_range = { 85, 99 },
  width = 198,
  height = 66,
  effects = { 'EFF_ZONE_MOUNT_DOOM' },

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { weighted = { SHALLOW_LAVA=90, A_BLAZING_FIRE=10 } },
      fill = { 'LAVA_WALL' },
      outer_wall = 'DEEP_LAVA',
      inner_wall = 'DARK_PIT',
      fill_step = 0,
      up = 'UP',
      down = 'DOWN',
      flags = {
	CAVE = true,
	LAVA_RIVER = true,
	NO_RECALL = true,
	NO_STREAMERS = true,
	NO_UP = true,
	NO_EASY_MOVE = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 14,
      respawn_in = 160,
      filters = {
	{ prob=100, mode='and', resist_dam={ DamageType.FIRE } },
      },
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=10, combat=10, magic=30, tools=30, junk=20 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_MORDOR',
	  -- up_lev = 32
	},
      },
    },
    [15] = {
      no_level_connectivity = true,
      level_name = [[Mt Doom]],
      enter_desc = [[You finally reach the top of Mount Doom, here must lie the Great Fire.]],
      generator = {
	map = {
	  class='mod.class.generator.map.Static',
	  map = 'zones/mount-doom',
	  persistent = true,
	  flags = { DESC=true, NO_GENO=true, NO_NEW_MONSTER=true, SPECIAL=true, NO_STAIR=true, NO_TELEPORT=true },
	  -- persist = mdm
	},
	actor = {
	  init_npc = table.NIL_MERGE,
	  respawn_in = table.NIL_MERGE,
	},
      },
    },
  },
}
