-- $Id: zone.lua 4279 2021-09-18 23:39:31Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2021 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- Zone file for Nether Realm.

local DamageType = require('engine.DamageType')

return {
  name = 'Nether Realm',
  display_abbr = 'Nth',
  level_scheme = 'fixed',
  max_level = 31,
  level_range = { 666, 696 },
  width = 198,
  height = 66,
  effects = { 'EFF_AIRLESS', 'EFF_FORGET_MAP', 'EFF_ZONE_NETHER_REALM' },

  generator = {
    map = {
      class = 'mod.class.generator.map.T2DungeonLevel',
      floor = { weighted = { NETHER_MIST=80, SHALLOW_LAVA=15, DEEP_LAVA=5 } },
      fill = { weighted = { DEEP_LAVA={80,50}, DARK_PIT={20,50} } },
      outer_wall = 'WALL',
      inner_wall = 'DEEP_LAVA',
      fill_step = 2,
      up = 'UP',
      down = 'DOWN',
      flags = {
	EMPTY = true,
	FORGET = true,
	NO_BREATH = true,
	NO_EASY_MOVE = true,
	NO_SHAFT = true,
	RANDOM_TOWNS = true,
	ADJUST_LEVEL_2 = true,
	NO_RECALL = true,
	NO_STREAMERS = true,
	LAVA_RIVER = true,
	NO_RECALL_OUT = true,
	NO_UP = true,
      },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      init_npc = 14,
      respawn_in = 160,
      filters = {
	{ prob=5, mode='any' },
	{ prob=95, mode='or', syms={ 'G', 'W', 'U' }, resist_dam={ DamageType.NETHER } },
      },
      guardian = 'TIK_SRVZLLAT',
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=25, combat=25, magic=25, tools=25, junk=0 },
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
    },
  },
  actor_adjust_level = function(zone, level, e)
    local lev = zone:level_adjust_level(level, 'actor')
    return lev*2
  end,
  alter_level_data = util.adjust_level,
  levels = {
    [1] = {
      generator = {
	map = {
	  up = 'UP_TO_VOID',
	  -- up_lev = 21
	},
      },
    },
  },
}
