-- $Id: zone.lua 4091 2020-09-23 02:04:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

return {
  name = "Fire Cave",
  level_scheme = 'fixed',
  max_level = 1,
  level_range = { 20, 20 },
  width = 49,
  height = 22,
  no_level_connectivity = true,
  no_recall_depth = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.Static',
      map = 'quests/lothlorien-fireproof',
      flags = { NO_TELEPORT=true },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      -- Actors are already placed by the Static map.
      no_prepop = true,
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=20, combat=20, magic=20, tools=20, junk=20 },
      no_prepop = true,	-- This is just for random objects on the map
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
      n_traps = { 10, 30 },
    }
  },

  post_process = function(level, zone)
    local cands = {}
    -- Pick a random essence type.
    for _, o in ipairs(game.zone.object_list) do
      if o.type == 'misc' and o.subtype == 'alchemic' and not o.special_gene then
	table.insert(cands, o.define_as)
      end
    end
    local cand = rng.table(cands)
    -- Create and place the essence.
    local ess = zone:makeEntityByName(level, 'object', cand)
    if ess then
      ess.quest_obj = 'lothlorien-fireproof'
      ess.quest_on_pickup = 'lothlorien-fireproof'
      ess:setFoundHere({ type='floor' }, zone, level)
      ess.on_drop = function(who, silent)
	if not silent then
	  game.logPlayer(who, [[You really should hang onto this until you've returned to the Wizards Spire.]])
	end
	return true
      end
      local x, y = rng.range(1, 45), rng.range(1, 3)
      print(('[QUEST] placing essence at %d,%d'):format(x, y))
      zone:addEntity(level, ess, 'object', x, y)
    else
      error(('[QUEST] failed to create essence %s (out of %d candidates)'):format(cand, #cands))
    end
  end,
}
