-- $Id: zone.lua 4091 2020-09-23 02:04:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

local Dialog = require 'engine.ui.Dialog'

return {
  name = "Wolves' Den",
  level_scheme = 'fixed',
  max_level = 1,
  level_range = { 15, 15 },
  width = 33,
  height = 23,
  no_level_connectivity = true,
  no_recall_depth = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.Static',
      map = 'quests/lothlorien-wolves',
      flags = { NO_GENO=true },
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      -- We'll generate actors in post_process() below.
      no_prepop = true,
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=20, combat=20, magic=20, tools=20, junk=20 },
      no_prepop = true,	-- This is just for random objects on the map
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
      -- We'll generate traps in post_process() below.
      n_traps = { 0, 0 },
    },
  },

  post_process = function(level, zone)
    local gen = zone:getGenerator('actor', level)
    -- Place some wolves.
    local n = rng.dice(4, 4)
    while n > 0 do
      local m = gen:generateOne{ name='WOLF', no_grp=true, sleep=50, ret=true }
      if m then
	m.quest_on_die = 'lothlorien-house'
	n = n - 1
      end
    end
    -- Place some wargs.
    n = rng.dice(4, 4)
    while n > 0 do
      local m = gen:generateOne{ name='WARG', no_grp=true, sleep=50, ret=true }
      if m then
	m.quest_on_die = 'lothlorien-house'
	n = n - 1
      end
    end
  end,
}
