-- $Id: zone.lua 4091 2020-09-23 02:04:38Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2020 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

return {
  name = 'Arda',
  level_scheme = 'fixed',
  max_level = 1,
  width = 101,
  height = 66,
  day_night = true,
  worldmap = true,
  no_level_connectivity = true,

  generator = {
    map = {
      class = 'mod.class.generator.map.Static',
      map = 'world'
    },
    actor = {
      class = 'mod.class.generator.actor.T2LevelFilteredActors',
      -- No random actors on the worldmap.
      no_prepop = true,
    },
    object = {
      class = 'mod.class.generator.object.T2LevelFilteredObjects',
      weights = { treasure=20, combat=20, magic=20, tools=20, junk=20 },
      no_prepop = true,	-- No random objects on the worldmap.
    },
    trap = {
      class = 'mod.class.generator.trap.T2LevelFilteredTraps',
      -- No random traps on the worldmap.
      n_traps = { 0, 0 },
    },
  },

  entry_point = function(_, _, _)
    -- We should only be entering the world map from a "wilderness" zone
    -- (including non-dungeon towns), which we can assume will have set
    -- game.player.wild_[xy] for us.
    return { x=game.player.wild_x, y=game.player.wild_y }
  end,

  on_leave = function(from_, to_lev, from_lev, to_zone, params)
    -- Remember our wilderness position.
    game.player.wild_x, game.player.wild_y = game.player.x, game.player.y
    -- Persist our seen/remembered state.
    if game.level then
      game.state.worldmap_seen = game.level.map:exportSeens()
      game.state.worldmap_remember = game.level.map:exportRemembers()
    end

    if util.zone_is(to_zone, 'dungeon') then
      -- Questionable Hack(TM):  If we're entering a dungeon zone from the
      -- worldmap, set town recall target to the wilderness level around
      -- the zone entrance.
      game.state:setRecallDest(to_zone)
    end
  end,

  post_process_map = function(level, zone)
    -- Restore seen/remembered state.
    level.map:importSeens(game.state.worldmap_seen or {})
    level.map:importRemembers(game.state.worldmap_remember or {})
    -- Make any grid modifications requested by wilderness quests.
    local mods = {}
    for _, loc in ipairs(game.state:getWildernessQuests()) do
      if loc.quest.worldmap_change_grid then
	local og = game.state:getWorldGrid(loc.x, loc.y)
	local g = loc.quest:worldmap_change_grid(x, y, og)
	if type(g) == 'string' then g = game.state.worldmap_grids[g] end
	if g then mods[#mods+1] = { x=loc.x, y=loc.y, g=g } end
      end
    end
    if #mods > 0 then
      game:onTickEnd(function()
	local Map = require 'mod.class.Map'
	for _, mod in ipairs(mods) do
	  level.map(mod.x, mod.y, Map.TERRAIN, mod.g)
	end
      end)
    end
  end,
}
