-- $Id: HotkeysDisplay.lua 2835 2017-04-11 00:13:59Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local HotkeysDisplay = require 'engine.HotkeysDisplay'

module(..., package.seeall, class.inherit(HotkeysDisplay))

function _M:toScreen()
  self:display()
  if self.bg_texture then
    self.bg_texture:toScreenFull(self.display_x, self.display_y, self.w, self.h, self.bg_texture_w, self.bg_texture_h)
  end
  for i = 1, #self.items do
    local item = self.items[i]
    local idx = item.i + 12*(item.page - 1)
    if self.cur_sel == idx then
      core.display.drawQuad(self.display_x + item.x, self.display_y + item.y, item.w, item.h, 0, 50, 120, 180)
    end
    if self.shadow then
      item._tex:toScreenFull(self.display_x + item.x + 2, self.display_y + item.y + 2, item.w, item.h, item._tex_w, item._tex_h, 0, 0, 0, self.shadow)
    end
    item._tex:toScreenFull(self.display_x + item.x, self.display_y + item.y, item.w, item.h, item._tex_w, item._tex_h)
  end
end

function _M:display()
  local a = self.actor
  if not a or not a.changed then return self.surface end

  local hks = {}
  for pg = 1, a.nb_hotkey_pages do
    for i = 1, 12 do
      local hk = a.hotkey[i + 12*(pg - 1)]
      if hk and (hk[1] == 'talent' or hk[1] == 'inventory' or hk[1] == 'thaumaturgy') then
	hks[#hks+1] = { hk[2], pg, i, hk[1] }
      end
    end
  end

  self.surface:erase(self.bgcolor[1], self.bgcolor[2], self.bgcolor[3])
  if self.bg_surface then self.surface:merge(self.bg_surface, 0, 0) end

  local x, y = 0, 0
  self.clics = {}
  self.items = {}

  for _, hk in ipairs(hks) do
    local pg, i = hk[2], hk[3]
    local txt, color = '', {0,255,0}

    if hk[4] == 'talent' then
      local t = a:getTalentFromId(hk[1])
      if a:isTalentCoolingDown(t) then
	txt = ("%s (%d)"):format(t.name, a:isTalentCoolingDown(t))
	color = {255,0,0}
      elseif a:isTalentActive(t.id) then
	txt = t.name
	color = {255,255,0}
      elseif not a:preUseTalent(t, true, true) then
	txt = t.name
	color = {190,190,190}
      else
	txt = t.name
	color = {0,255,0}
      end
    elseif hk[4] == 'thaumaturgy' then
      local sp = a.thaumaturgy_spells[hk[1]]
      txt = sp.name
      local t = spell.thaumaturgy.stub_talent(sp)
      if not a:preUseTalent(t, true, true) then
	color = {190,190,190}
      else
	color = {0,255,0}
      end
    elseif hk[4] == 'inventory' then
      local o = a:findInAllInventories(hk[1], {no_add_name=true, force_id=true, no_count=true})
      local cnt = o and o:getNumber() or 0
      txt = ("%s (%d)"):format(o and o:getName{no_count=true} or hk[1], cnt)
      if cnt == 0 then color = {128,128,128} end
    end

    txt = ("%1d/%2d) %s"):format(pg, i, txt)
    local w, h, gen
    if self.cache[txt] then
      gen = self.cache[txt]
      w, h = gen.fw, gen.fh
    else
      w, h = self.font:size(txt)
      gen = self.font:draw(txt, self.w / self.nb_cols, color[1], color[2], color[3], true)[1]
      gen.fw, gen.fh = w, h
    end
    gen.x, gen.y = x, y
    gen.page, gen.i = pg, i
    self.items[#self.items+1] = gen
    self.clics[i + (12 * (pg - 1))] = {x,y,w+4,h+4}

    if y + self.font_h * 2 > self.h then
      x = x + self.w / self.nb_cols
      y = 0
    else
      y = y + self.font_h
    end
  end
end

--- Call when a mouse event arrives in this zone
-- This is optional, only if you need mouse support
function _M:onMouse(button, mx, my, click, on_over, on_click)
  local a = self.actor

  if button == "wheelup" and click then
    a:prevHotkeyPage()
    return
  elseif button == "wheeldown" and click then
    a:nextHotkeyPage()
    return
  end

  mx, my = mx - self.display_x, my - self.display_y
  for i, zone in pairs(self.clics) do
    if mx >= zone[1] and mx < zone[1] + zone[3] and my >= zone[2] and my < zone[2] + zone[4] then
      if on_click and click then
	if on_click(i, a.hotkey[i]) then click = false end
      end
      if button == "left" and click then
	a:activateHotkey(i)
      elseif button == "right" and click then
	a.hotkey[i] = nil
	a.changed = true
      else
	a.changed = true
	local oldsel = self.cur_sel
	self.cur_sel = i
	if on_over and self.cur_sel ~= oldsel then
	  local hk = a.hotkey[i]
	  local text = ""
	  if hk and hk[1] == "talent" then
	    local t = self.actor:getTalentFromId(hk[2])
	    text = tstring{{"color","GOLD"}, {"font", "bold"}, t.name, {"font", "normal"}, {"color", "LAST"}, true}
	    text:merge(self.actor:getTalentFullDescription(t))
	  elseif hk and hk[1] == 'thaumaturgy' then
	    local sp = a.thaumaturgy_spells[hk[2]]
	    local t = spell.thaumaturgy.stub_talent(sp)
	    text = tstring{{"color","GOLD"}, {"font", "bold"}, sp.name, {"font", "normal"}, {"color", "LAST"}, true}
	    text:merge(self.actor:getTalentFullDescription(t))
	  elseif hk and hk[1] == "inventory" then
	    local o = a:findInAllInventories(hk[2])
	    if o then text = o:getDesc() else text = "Missing!" end
	  end
	  on_over(text)
	end
      end
      return
    end
  end
  self.cur_sel = nil
end
