-- $Id: VisualsDialog.lua 3635 2019-07-15 04:08:23Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Map = require 'mod.class.Map'
local Entity = require 'engine.Entity'
local Base = require 'engine.ui.Base'
local Dialog = require 'engine.ui.Dialog'
local Button = require 'engine.ui.Button'
local List = require 'engine.ui.List'
local Textzone = require 'engine.ui.Textzone'
local TextzoneList = require 'engine.ui.TextzoneList'
local OpaqueDropdown = require 'mod.dialogs.ui.OpaqueDropdown'
local Separator = require 'engine.ui.Separator'
local ActorFrame = require 'engine.ui.ActorFrame'
local TilesList = require 'mod.dialogs.ui.TilesList'

module(..., package.seeall, class.inherit(Dialog))

_M.colors_list = {
  { name='#202020#Black#LAST#',   color=colors_simple.C_d, color_hex='000000' },
  { name='#C__W#White#LAST#',     color=colors_simple.C_w, color_hex='FFFFFF' },
  { name='#C__S#Gray#LAST#',      color=colors_simple.C_s, color_hex='808080' },
  { name='#C__O#Orange#LAST#',    color=colors_simple.C_o, color_hex='FF8000' },
  { name='#C__R#Red#LAST#',       color=colors_simple.C_r, color_hex='C00000' },
  { name='#C__G#Green#LAST#',     color=colors_simple.C_g, color_hex='008000' },
  { name='#C__B#Blue#LAST#',      color=colors_simple.C_b, color_hex='0000FF' },
  { name='#C__U#Brown#LAST#',     color=colors_simple.C_u, color_hex='804000' },
  { name='#C_D#Dark Gray#LAST#',  color=colors_simple.C_D, color_hex='404040' },
  { name='#C_W#Light Gray#LAST#', color=colors_simple.C_W, color_hex='C0C0C0' },
  { name='#C__V#Violet#LAST#',    color=colors_simple.C_v, color_hex='FF00FF' },
  { name='#C__Y#Yellow#LAST#',    color=colors_simple.C_y, color_hex='FFFF00' },
  { name='#C_R#Light Red#LAST#',  color=colors_simple.C_R, color_hex='FF0000' },
  { name='#C_G#Light Green#LAST#', color=colors_simple.C_G, color_hex='00FF00' },
  { name='#C_B#Light Blue#LAST#', color=colors_simple.C_B, color_hex='00FFFF' },
  { name='#C_U#Light Brown#LAST#', color=colors_simple.C_U, color_hex='C08040' },
}
-- This is just the list of available code points scraped from the engine's
-- Droid Sans Mono font, as the engine doesn't appear to give us a way to
-- query the font for this information.
_M.tiles_list = {
  [[ ]], [[!]], [["]], [[#]], [[$]], [[%]], [[&]], [[']],
  [[(]], [[)]], [[*]], [[+]], [[,]], [[-]], [[.]], [[/]],
  [[0]], [[1]], [[2]], [[3]], [[4]], [[5]], [[6]], [[7]],
  [[8]], [[9]], [[:]], [[;]], [[<]], [[=]], [[>]], [[?]],
  [[@]], [[A]], [[B]], [[C]], [[D]], [[E]], [[F]], [[G]],
  [[H]], [[I]], [[J]], [[K]], [[L]], [[M]], [[N]], [[O]],
  [[P]], [[Q]], [[R]], [[S]], [[T]], [[U]], [[V]], [[W]],
  [[X]], [[Y]], [[Z]], '[', [[\]], ']', [[^]], [[_]],
  [[`]], [[a]], [[b]], [[c]], [[d]], [[e]], [[f]], [[g]],
  [[h]], [[i]], [[j]], [[k]], [[l]], [[m]], [[n]], [[o]],
  [[p]], [[q]], [[r]], [[s]], [[t]], [[u]], [[v]], [[w]],
  [[x]], [[y]], [[z]], [[{]], [[|]], [[}]], [[~]], [[ ]],
  [[¡]], [[¢]], [[£]], [[¤]], [[¥]], [[¦]], [[§]], [[¨]],
  [[©]], [[ª]], [[«]], [[¬]], [[­]], [[®]], [[¯]], [[°]],
  [[±]], [[²]], [[³]], [[´]], [[µ]], [[¶]], [[·]], [[¸]],
  [[¹]], [[º]], [[»]], [[¼]], [[½]], [[¾]], [[¿]], [[À]],
  [[Á]], [[Â]], [[Ã]], [[Ä]], [[Å]], [[Æ]], [[Ç]], [[È]],
  [[É]], [[Ê]], [[Ë]], [[Ì]], [[Í]], [[Î]], [[Ï]], [[Ð]],
  [[Ñ]], [[Ò]], [[Ó]], [[Ô]], [[Õ]], [[Ö]], [[×]], [[Ø]],
  [[Ù]], [[Ú]], [[Û]], [[Ü]], [[Ý]], [[Þ]], [[ß]], [[à]],
  [[á]], [[â]], [[ã]], [[ä]], [[å]], [[æ]], [[ç]], [[è]],
  [[é]], [[ê]], [[ë]], [[ì]], [[í]], [[î]], [[ï]], [[ð]],
  [[ñ]], [[ò]], [[ó]], [[ô]], [[õ]], [[ö]], [[÷]], [[ø]],
  [[ù]], [[ú]], [[û]], [[ü]], [[ý]], [[þ]], [[ÿ]], [[Ā]],
  [[ā]], [[Ă]], [[ă]], [[Ą]], [[ą]], [[Ć]], [[ć]], [[Ĉ]],
  [[ĉ]], [[Ċ]], [[ċ]], [[Č]], [[č]], [[Ď]], [[ď]], [[Đ]],
  [[đ]], [[Ē]], [[ē]], [[Ĕ]], [[ĕ]], [[Ė]], [[ė]], [[Ę]],
  [[ę]], [[Ě]], [[ě]], [[Ĝ]], [[ĝ]], [[Ğ]], [[ğ]], [[Ġ]],
  [[ġ]], [[Ģ]], [[ģ]], [[Ĥ]], [[ĥ]], [[Ħ]], [[ħ]], [[Ĩ]],
  [[ĩ]], [[Ī]], [[ī]], [[Ĭ]], [[ĭ]], [[Į]], [[į]], [[İ]],
  [[ı]], [[Ĳ]], [[ĳ]], [[Ĵ]], [[ĵ]], [[Ķ]], [[ķ]], [[ĸ]],
  [[Ĺ]], [[ĺ]], [[Ļ]], [[ļ]], [[Ľ]], [[ľ]], [[Ŀ]], [[ŀ]],
  [[Ł]], [[ł]], [[Ń]], [[ń]], [[Ņ]], [[ņ]], [[Ň]], [[ň]],
  [[ŉ]], [[Ŋ]], [[ŋ]], [[Ō]], [[ō]], [[Ŏ]], [[ŏ]], [[Ő]],
  [[ő]], [[Œ]], [[œ]], [[Ŕ]], [[ŕ]], [[Ŗ]], [[ŗ]], [[Ř]],
  [[ř]], [[Ś]], [[ś]], [[Ŝ]], [[ŝ]], [[Ş]], [[ş]], [[Š]],
  [[š]], [[Ţ]], [[ţ]], [[Ť]], [[ť]], [[Ŧ]], [[ŧ]], [[Ũ]],
  [[ũ]], [[Ū]], [[ū]], [[Ŭ]], [[ŭ]], [[Ů]], [[ů]], [[Ű]],
  [[ű]], [[Ų]], [[ų]], [[Ŵ]], [[ŵ]], [[Ŷ]], [[ŷ]], [[Ÿ]],
  [[Ź]], [[ź]], [[Ż]], [[ż]], [[Ž]], [[ž]], [[ſ]], [[ƒ]],
  [[Ơ]], [[ơ]], [[Ư]], [[ư]], [[ǰ]], [[Ǻ]], [[ǻ]], [[Ǽ]],
  [[ǽ]], [[Ǿ]], [[ǿ]], [[Ș]], [[ș]], [[ʼ]], [[ˆ]], [[ˇ]],
  [[ˉ]], [[˘]], [[˙]], [[˚]], [[˛]], [[˜]], [[˝]], [[˳]],
  [[̀]], [[́]], [[̃]], [[̉]], [[̏]], [[̣]], [[΄]], [[΅]],
  [[Ά]], [[·]], [[Έ]], [[Ή]], [[Ί]], [[Ό]], [[Ύ]], [[Ώ]],
  [[ΐ]], [[Α]], [[Β]], [[Γ]], [[Δ]], [[Ε]], [[Ζ]], [[Η]],
  [[Θ]], [[Ι]], [[Κ]], [[Λ]], [[Μ]], [[Ν]], [[Ξ]], [[Ο]],
  [[Π]], [[Ρ]], [[Σ]], [[Τ]], [[Υ]], [[Φ]], [[Χ]], [[Ψ]],
  [[Ω]], [[Ϊ]], [[Ϋ]], [[ά]], [[έ]], [[ή]], [[ί]], [[ΰ]],
  [[α]], [[β]], [[γ]], [[δ]], [[ε]], [[ζ]], [[η]], [[θ]],
  [[ι]], [[κ]], [[λ]], [[μ]], [[ν]], [[ξ]], [[ο]], [[π]],
  [[ρ]], [[ς]], [[σ]], [[τ]], [[υ]], [[φ]], [[χ]], [[ψ]],
  [[ω]], [[ϊ]], [[ϋ]], [[ό]], [[ύ]], [[ώ]], [[ϑ]], [[ϒ]],
  [[ϖ]], [[Ѐ]], [[Ё]], [[Ђ]], [[Ѓ]], [[Є]], [[Ѕ]], [[І]],
  [[Ї]], [[Ј]], [[Љ]], [[Њ]], [[Ћ]], [[Ќ]], [[Ѝ]], [[Ў]],
  [[Џ]], [[А]], [[Б]], [[В]], [[Г]], [[Д]], [[Е]], [[Ж]],
  [[З]], [[И]], [[Й]], [[К]], [[Л]], [[М]], [[Н]], [[О]],
  [[П]], [[Р]], [[С]], [[Т]], [[У]], [[Ф]], [[Х]], [[Ц]],
  [[Ч]], [[Ш]], [[Щ]], [[Ъ]], [[Ы]], [[Ь]], [[Э]], [[Ю]],
  [[Я]], [[а]], [[б]], [[в]], [[г]], [[д]], [[е]], [[ж]],
  [[з]], [[и]], [[й]], [[к]], [[л]], [[м]], [[н]], [[о]],
  [[п]], [[р]], [[с]], [[т]], [[у]], [[ф]], [[х]], [[ц]],
  [[ч]], [[ш]], [[щ]], [[ъ]], [[ы]], [[ь]], [[э]], [[ю]],
  [[я]], [[ѐ]], [[ё]], [[ђ]], [[ѓ]], [[є]], [[ѕ]], [[і]],
  [[ї]], [[ј]], [[љ]], [[њ]], [[ћ]], [[ќ]], [[ѝ]], [[ў]],
  [[џ]], [[Ѡ]], [[ѡ]], [[Ѣ]], [[ѣ]], [[Ѥ]], [[ѥ]], [[Ѧ]],
  [[ѧ]], [[Ѩ]], [[ѩ]], [[Ѫ]], [[ѫ]], [[Ѭ]], [[ѭ]], [[Ѯ]],
  [[ѯ]], [[Ѱ]], [[ѱ]], [[Ѳ]], [[ѳ]], [[Ѵ]], [[ѵ]], [[Ѷ]],
  [[ѷ]], [[Ѹ]], [[ѹ]], [[Ѻ]], [[ѻ]], [[Ѽ]], [[ѽ]], [[Ѿ]],
  [[ѿ]], [[Ҁ]], [[ҁ]], [[҂]], [[҃]], [[҄]], [[҅]], [[҆]],
  [[҈]], [[҉]], [[Ҋ]], [[ҋ]], [[Ҍ]], [[ҍ]], [[Ҏ]], [[ҏ]],
  [[Ґ]], [[ґ]], [[Ғ]], [[ғ]], [[Ҕ]], [[ҕ]], [[Җ]], [[җ]],
  [[Ҙ]], [[ҙ]], [[Қ]], [[қ]], [[Ҝ]], [[ҝ]], [[Ҟ]], [[ҟ]],
  [[Ҡ]], [[ҡ]], [[Ң]], [[ң]], [[Ҥ]], [[ҥ]], [[Ҧ]], [[ҧ]],
  [[Ҩ]], [[ҩ]], [[Ҫ]], [[ҫ]], [[Ҭ]], [[ҭ]], [[Ү]], [[ү]],
  [[Ұ]], [[ұ]], [[Ҳ]], [[ҳ]], [[Ҵ]], [[ҵ]], [[Ҷ]], [[ҷ]],
  [[Ҹ]], [[ҹ]], [[Һ]], [[һ]], [[Ҽ]], [[ҽ]], [[Ҿ]], [[ҿ]],
  [[Ӏ]], [[Ӂ]], [[ӂ]], [[Ӄ]], [[ӄ]], [[Ӆ]], [[ӆ]], [[Ӈ]],
  [[ӈ]], [[Ӊ]], [[ӊ]], [[Ӌ]], [[ӌ]], [[Ӎ]], [[ӎ]], [[ӏ]],
  [[Ӑ]], [[ӑ]], [[Ӓ]], [[ӓ]], [[Ӕ]], [[ӕ]], [[Ӗ]], [[ӗ]],
  [[Ә]], [[ә]], [[Ӛ]], [[ӛ]], [[Ӝ]], [[ӝ]], [[Ӟ]], [[ӟ]],
  [[Ӡ]], [[ӡ]], [[Ӣ]], [[ӣ]], [[Ӥ]], [[ӥ]], [[Ӧ]], [[ӧ]],
  [[Ө]], [[ө]], [[Ӫ]], [[ӫ]], [[Ӭ]], [[ӭ]], [[Ӯ]], [[ӯ]],
  [[Ӱ]], [[ӱ]], [[Ӳ]], [[ӳ]], [[Ӵ]], [[ӵ]], [[Ӷ]], [[ӷ]],
  [[Ӹ]], [[ӹ]], [[Ӻ]], [[ӻ]], [[Ӽ]], [[ӽ]], [[Ӿ]], [[ӿ]],
  [[Ԁ]], [[ԁ]], [[Ԃ]], [[ԃ]], [[Ԅ]], [[ԅ]], [[Ԇ]], [[ԇ]],
  [[Ԉ]], [[ԉ]], [[Ԋ]], [[ԋ]], [[Ԍ]], [[ԍ]], [[Ԏ]], [[ԏ]],
  [[Ԑ]], [[ԑ]], [[Ԓ]], [[ԓ]], [[Ḁ]], [[ḁ]], [[Ḿ]], [[ḿ]],
  [[Ẁ]], [[ẁ]], [[Ẃ]], [[ẃ]], [[Ẅ]], [[ẅ]], [[Ạ]], [[ạ]],
  [[Ả]], [[ả]], [[Ấ]], [[ấ]], [[Ầ]], [[ầ]], [[Ẩ]], [[ẩ]],
  [[Ẫ]], [[ẫ]], [[Ậ]], [[ậ]], [[Ắ]], [[ắ]], [[Ằ]], [[ằ]],
  [[Ẳ]], [[ẳ]], [[Ẵ]], [[ẵ]], [[Ặ]], [[ặ]], [[Ẹ]], [[ẹ]],
  [[Ẻ]], [[ẻ]], [[Ẽ]], [[ẽ]], [[Ế]], [[ế]], [[Ề]], [[ề]],
  [[Ể]], [[ể]], [[Ễ]], [[ễ]], [[Ệ]], [[ệ]], [[Ỉ]], [[ỉ]],
  [[Ị]], [[ị]], [[Ọ]], [[ọ]], [[Ỏ]], [[ỏ]], [[Ố]], [[ố]],
  [[Ồ]], [[ồ]], [[Ổ]], [[ổ]], [[Ỗ]], [[ỗ]], [[Ộ]], [[ộ]],
  [[Ớ]], [[ớ]], [[Ờ]], [[ờ]], [[Ở]], [[ở]], [[Ỡ]], [[ỡ]],
  [[Ợ]], [[ợ]], [[Ụ]], [[ụ]], [[Ủ]], [[ủ]], [[Ứ]], [[ứ]],
  [[Ừ]], [[ừ]], [[Ử]], [[ử]], [[Ữ]], [[ữ]], [[Ự]], [[ự]],
  [[Ỳ]], [[ỳ]], [[Ỵ]], [[ỵ]], [[Ỷ]], [[ỷ]], [[Ỹ]], [[ỹ]],
  [[Ὅ]], [[ ]], [[ ]], [[ ]], [[ ]], [[ ]], [[ ]], [[ ]],
  [[ ]], [[ ]], [[ ]], [[ ]], [[​]], [[–]], [[—]], [[―]],
  [[‗]], [[‘]], [[’]], [[‚]], [[‛]], [[“]], [[”]], [[„]],
  [[†]], [[‡]], [[•]], [[…]], [[‰]], [[′]], [[″]], [[‹]],
  [[›]], [[‼]], [[⁄]], [[ⁿ]], [[₣]], [[₤]], [[₧]], [[₫]],
  [[€]], [[℅]], [[ℓ]], [[№]], [[™]], [[Ω]], [[℮]], [[⅛]],
  [[⅜]], [[⅝]], [[⅞]], [[∂]], [[∆]], [[∏]], [[∑]], [[−]],
  [[√]], [[∞]], [[∫]], [[≈]], [[≠]], [[≤]], [[≥]], [[◊]],
  [[ﬁ]], [[ﬂ]], [[ﬃ]], [[ﬄ]], [[﻿]], [[￼]], [[�]],
}

function _M:init(title, entities, init_mods, name_cb)
  self.entities = entities
  self.init_mods = init_mods
  self.mods = table.clone(init_mods, true)
  self.name_cb = name_cb
  self:generateList()

  Dialog.init(self, title, 1, math.floor(game.h*0.8))

  self.c_def_lbl = Textzone.new {
    text = 'default',
    auto_height = true,
    auto_width = true,
  }
  self.c_cur_lbl = Textzone.new {
    text = 'current',
    auto_height = true,
    auto_width = true,
  }
  -- generate() these to get their widths and heights
  self.c_def_lbl:generate()
  self.c_cur_lbl:generate()
  self.c_colors = OpaqueDropdown.new {
    width = 100,
    list = self.colors_list,
    nb_items = #self.colors_list,
    fct = function(item) self:updateMod() end,
    alpha = 1,
  }
  local tiles_w = (Map.tile_w + 2)*16 + 10
  self.c_desc = TextzoneList.new {
    width = tiles_w,
    height = self.c_cur_lbl.h,
    weakstore = true,
  }
  self.c_entities = List.new {
    width = self.max_w + 15,
    height = self.ih - 10,
    list = self.list,
    scrollbar = true,
    fct = function(item) self:selectEntity(item) end,
  }
  self.c_ok = Button.new {
    text = 'OK',
    fct = function() game:unregisterDialog(self) end,
  }
  self.c_cancel = Button.new {
    text = 'Cancel',
    fct = function() self.key:triggerVirtual('EXIT') end,
  }
  self.c_reset = Button.new {
    text = 'Reset to default',
    fct = function() self:clearMod(self.cur_entity) end,
  }
  -- generate() these to get their widths and heights
  self.c_ok:generate()
  self.c_cancel:generate()
  self.c_reset:generate()
  local cdr_h = math.max(self.c_cur_lbl.h, Map.tile_h)
  local cldrb_h = math.max(self.c_colors.font_h + 6, self.c_reset.h)
  local tiles_h = self.ih - self.c_desc.h - cdr_h - cldrb_h - Map.tile_h - self.c_ok.h - 22
  tiles_h = tiles_h - (tiles_h % (Map.tile_h+2))
  self.c_tiles = TilesList.new {
    tile_w = Map.tile_w,
    tile_h = Map.tile_h,
    width = tiles_w,
    height = tiles_h,
    list = self.tiles_list,
    fct = function(item) self:updateMod() end,
    padding = 2,
    selection = 'simple',
    fontname = Map.fontname,
    fontsize = Map.fontsize,
    scrollbar = true,
  }
  self.c_def_tile = ActorFrame.new {
    actor = {},
    w = Map.tile_w,
    h = Map.tile_h,
    tiles = self.c_tiles.tiles,
  }
  self.c_cur_tile = ActorFrame.new {
    actor = {},
    w = Map.tile_w,
    h = Map.tile_h,
    tiles = self.c_tiles.tiles,
  }
  local vs = Separator.new { dir='horizontal', size=self.ih }

  local h1 = self.c_entities.w + 10
  local h2 = h1 + vs.w + 10
  local h_cdr1 = h2 + self.c_def_lbl.w + 10
  local h_cdr2 = h_cdr1 + self.c_def_tile.w + 10
  local h_cdr3 = h_cdr2 + self.c_cur_lbl.w + 10
  local h_cldrb_1 = h2 + self.c_colors.w + 10
  local h_b1 = h2 + tiles_w/2 - self.c_ok.w - 5
  local h_b2 = h2 + tiles_w/2 + 5
  local v1 = self.c_desc.h + 5
  local cdr_lbl_dy = math.floor((cdr_h - self.c_cur_lbl.h)/2)
  local cdr_tile_dy = math.floor((cdr_h - self.c_cur_tile.h)/2)
  local v2 = v1 + cdr_h + 5
  local cldrb_drp_dy = math.floor((cldrb_h - self.c_colors.font_h - 6)/2)
  local cldrb_btn_dy = math.floor((cldrb_h - self.c_reset.h)/2)
  local v3 = v2 + cldrb_h + Map.tile_h/2 + 6
  self:loadUI {
    { left=0, top=0, ui=self.c_entities },
    { left=h1, top=0, ui=vs },
    { left=h2, top=0, ui=self.c_desc },
    { left=h2, top=v1+cdr_lbl_dy, ui=self.c_def_lbl },
    { left=h_cdr1, top=v1+cdr_tile_dy, ui=self.c_def_tile },
    { left=h_cdr2, top=v1+cdr_lbl_dy, ui=self.c_cur_lbl },
    { left=h_cdr3, top=v1+cdr_tile_dy, ui=self.c_cur_tile },
    { left=h2, top=v2+cldrb_drp_dy, ui=self.c_colors },
    { left=h_cldrb_1, top=v2+cldrb_btn_dy, ui=self.c_reset },
    { left=h2, top=v3, ui=self.c_tiles },
    { left=h_b1, bottom=0, ui=self.c_ok },
    { left=h_b2, bottom=0, ui=self.c_cancel },
  }
  self:setupUI(true, false)

  self.key:addCommands {
    __TEXTINPUT = function(c)
      for i = self.c_entities.sel+1, #self.list do
	if self.list[i].name:sub(1, 1):lower() == c:lower() then
	  self.c_entities:select(i)
	  return
	end
      end
      for i = 1, self.c_entities.sel do
	if self.list[i].name:sub(1, 1):lower() == c:lower() then
	  self.c_entities:select(i)
	  return
	end
      end
    end
  }
  self.key:addBinds {
    EXIT = function()
      self.mods = nil
      game:unregisterDialog(self)
    end
  }
end

function _M:on_register()
  game:onTickEnd(function() self.key:unicodeInput(true) end)
end

-- Static
function _M:create(tag, init_val, name_cb)
  local vals = {
    grid = { 'Terrain Visuals', 'grid_list' },
    actor = { 'Monster Visuals', 'npc_list' },
    object = { 'Object Visuals', 'object_list' },
  }
  local name_cbs = {
    grid = function(e)
      return e.define_as ~= 'NOTHING' and e.name or nil
    end,
    actor = function(e)
      return e.define_as ~= 'Player' and e.name or nil
    end,
    object = function(e)
      return e:getName{no_count=true, force_id=true}
    end,
  }
  local key = 'ascii_'..tag..'_symbols'
  return self.new(vals[tag][1], game.zone[vals[tag][2]], init_val or config.settings.tome2[key], name_cb or name_cbs[tag])
end

function _M:get_value()
  return self.mods
end

function _M:get_display_val()
  return self:displayValue(self.mods)
end

function _M:displayValue(mods)
  if not mods then return nil end
  local n = 0
  for _, _ in pairs(mods) do n = n + 1 end
  if n == 0 then return 'no overrides' end
  if n == 1 then return '1 override' end
  return ('%d overrides'):format(n)
end

function _M:selectEntity(item)
  if item.uid then
    local by_id = self.items_by_id[item.define_as]
    if by_id then
      self.c_entities:select(by_id.idx)
      self:selectEntity(by_id.item)
    end
    return
  end
  if not item then return end
  self.cur_entity = item
  self.c_desc:switchItem(item.name, item.name)
  local mod = self.mods[item.e.define_as] or self:defaultMod(item)
  local i, _ = self:findColor(mod.color)
  if i then self.c_colors.c_list.sel = i end
  self.c_tiles:clearSelection()
  for _, row in ipairs(self.c_tiles.dlist) do
    for _, tile in ipairs(row) do
      if tile.text == item.e.display then
	tile.selected = true
	break
      end
    end
  end
  self:updateTiles()
end

function _M:findColor(rgb)
  if rgb.color_r then
    rgb = { r=rgb.color_r, g=rgb.color_g, b=rgb.color_b }
  end
  for i, d in ipairs(self.colors_list) do
    if d.color[1] == rgb.r and d.color[2] == rgb.g and d.color[3] == rgb.b then
      return i, d.color_hex
    end
  end
end

function _M:updateTiles()
  local item = self.cur_entity
  if not item then return end
  local mod = self.mods[item.e.define_as] or self:defaultMod(item)
  self:updateTile(self.c_def_tile, item.e.display, item.e)
  self:updateTile(self.c_cur_tile, mod.display, mod.color)
end

function _M:updateTile(c, display, color)
  if color.color_r then
    -- Translate Entity's separated colors back into a single color{}.
    color = { r=color.color_r, g=color.color_g, b=color.color_b }
  end
  if not color.r and #color == 3 then
    -- ??? Where are we getting a color_simple{} format from?  Convert it.
    color = { r=color[1], g=color[2], b=color[3] }
  end
  c.actor = Entity.new { display=display, color=color }
  c.actor.image = true
end

function _M:updateMod()
  local e_item = self.cur_entity
  local c_item = self.c_colors.c_list.list[self.c_colors.c_list.sel]
  local t_item = self.c_tiles:getAllSelected()[1]
  if not e_item or not c_item or not t_item then return end
  local color = { r=c_item.color[1], g=c_item.color[2], b=c_item.color[3] }
  self.mods[e_item.e.define_as] = { display=t_item.text, color=color }
  self:updateTiles()
  e_item.color = self:isModChanged(e_item) and { 0, 255, 0 } or nil
  self.c_entities:drawItem(e_item)
end

function _M:isModChanged(item)
  if not item then return false end
  local mod = self.mods[item.e.define_as]
  return mod and (mod.display ~= item.e.display or
		  mod.color.r ~= item.e.color_r or
		  mod.color.g ~= item.e.color_g or
		  mod.color.b ~= item.e.color_b)
end

function _M:defaultMod(item)
  return {
    display = item.e.display,
    color = { r=item.e.color_r, g=item.e.color_g, b=item.e.color_b },
  }
end

function _M:clearMod(item)
  if not item then return end
  self.mods[item.e.define_as] = nil
  item.color = nil
  self.c_entities:drawItem(item)
  self:updateTiles()
end

function _M:generateList()
  local list = {}
  local items_by_id = {}
  self.max_w = 0

  for _, e in ipairs(self.entities) do
    local name = e.name
    if self.name_cb then name = self.name_cb(e) end
    if name and not e.looks_like then
      local item = { name=name, e=e }
      item.color = self:isModChanged(item) and { 0, 255, 0 } or nil
      list[#list+1] = item
      items_by_id[e.define_as] = { item=item, idx=#list }
      local w, _ = self.font:size(name)
      self.max_w = math.max(self.max_w, w)
    end
  end

  self.list = list
  self.items_by_id = items_by_id
end

local function update_entity(e, mods, disp_class)
  if e.define_as then
    local mod = mods[e.looks_like or e.define_as]
    local rd = nil
    if mod and disp_class then
      rd = disp_class.new {
	display = mod.display,
	color = mod.color,
	-- Sneaky Hack(TM):  Use the entity's original image in our
	-- user-pref-derived replace_entity, so that our mod will
	-- effectively have no effect in non-ASCII mode.
	image = e.image,
	cosmetic = true,
      }
    end
    e:replaceDisplay('user-pref', rd)
  end
end

local entity_classes = {
  grid = require 'mod.class.Grid',
  actor = require 'mod.class.Actor',
  object = require 'mod.class.Object',
}

-- Static
function _M:updateEntity(e, tag)
  tag = tag or e:getEntityKind()
  local key = 'ascii_'..tag..'_symbols'
  local mods = config.settings.tome2[key]
  if entity_classes[tag] then
    update_entity(e, mods, entity_classes[tag])
  end
end

-- Static
function _M:updateEntityList(entities, mods, disp_class)
  disp_class = disp_class or entity_classes[entities[1]:getEntityKind()]
  for _, e in ipairs(entities) do update_entity(e, mods, disp_class) end
end

-- Static
function _M:updateActorInventory(a)
  local mods = config.settings.tome2.ascii_object_symbols or {}
  local Object = require 'mod.class.Object'
  local function cb(_, _, o) update_entity(o, mods, Object) end
  a:inventoryApplyAll(cb)
end

-- Static
function _M:updateLevel(t)
  t = t or { grid=true, actor=true, object=true }

  local g_mods = config.settings.tome2.ascii_grid_symbols or {}
  local a_mods = config.settings.tome2.ascii_actor_symbols or {}
  local o_mods = config.settings.tome2.ascii_object_symbols or {}
  local Grid = require 'mod.class.Grid'
  local Actor = require 'mod.class.Actor'
  local Object = require 'mod.class.Object'

  if t.grid then
    -- Check the map for cloned Grid tiles that won't get updated by an
    -- updateEntityList() on the zone's grid list.
    for x = 0, game.level.map.w - 1 do
      for y = 0, game.level.map.h - 1 do
	local g = game.level.map(x, y, Map.TERRAIN)
	local gg = game.zone.grid_list[g.define_as]
	-- Check for different user-pref mods.
	local g_rd = g.replace_displays and g.replace_displays['user-pref']
	local gg_rd = gg.replace_displays and gg.replace_displays['user-pref']
	if not table.equal(g_rd, gg_rd) then
	  g:removeAllMOs()
	  update_entity(g, g_mods, Grid)
	end
      end
    end
  end

  if t.actor then
    -- Update actors and their inventories.
    for _, e in pairs(game.level.entities) do
      e:removeAllMOs()
      update_entity(e, a_mods, Actor)
      if e.inventoryApplyAll and t.object then
	self:updateActorInventory(e)
      end
    end
  end

  if t.actor then
    -- Update floor objects.
    for x = 0, game.level.map.w - 1 do
      for y = 0, game.level.map.h - 1 do
	local n = game.level.map:getObjectTotal(x, y)
	if n > 0 then
	  for i = 1, n do
	    local o = game.level.map:getObject(x, y, i)
	    o:removeAllMOs()
	    update_entity(o, o_mods, Object)
	  end
	end
      end
    end
  end
end

-- Static
function _M:updateForEntityType(tag, mods)
  if tag == 'grid' then
    self:updateEntityList(game.zone.grid_list, mods, require('mod.class.Grid'))
    self:updateLevel{grid=true}
  elseif tag == 'actor' then
    self:updateEntityList(game.zone.npc_list, mods, require('mod.class.Actor'))
    self:updateLevel{actor=true}
  elseif tag == 'object' then
    self:updateEntityList(game.zone.object_list, mods, require('mod.class.Object'))
    self:updateLevel{object=true}
  end
end
