-- $Id: CreateItemDialog.lua 4277 2021-09-18 03:21:42Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2012-2017 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Dialog = require 'engine.ui.Dialog'
local List = require 'engine.ui.List'
local Button = require 'engine.ui.Button'
local Numberbox = require 'engine.ui.Numberbox'
local Checkbox = require 'engine.ui.Checkbox'
local Textbox = require 'engine.ui.Textbox'

module(..., package.seeall, class.inherit(Dialog))

function _M:init()
  self.ego_names = {}

  self:generateGroupList()

  local radio_grps = {
    id = { 'c_no_id', 'c_id', 'c_full_id' },
    power = { 'c_good', 'c_great', 'c_power' },
  }
  local function radio(grp, selected)
    for _, cbox in ipairs(radio_grps[grp] or {}) do
      self[cbox].checked = (cbox == selected)
    end
  end

  self.c_groups = List.new {
    width = 250,
    nb_items = 10,
    display_prop = 'display',
    list = self.groups,
    scrollbar = true,
    fct = function(item) self:setupForGroup(item) end,
  }
  self.c_items = List.new {
    width = 400,
    nb_items = 10,
    display_prop = 'display',
    list = {},
    scrollbar = true,
    fct = function(item) self:selectItem(item) end,
  }
  self.c_pfx_egos = List.new {
    width = 200,
    nb_items = 10,
    display_prop = 'display',
    list = {},
    scrollbar = true,
    fct = function(item) self:selectEgo(item, self.c_pfx_egos, 'prefix') end,
  }
  self.c_sfx_egos = List.new {
    width = 200,
    nb_items = 10,
    display_prop = 'display',
    list = {},
    scrollbar = true,
    fct = function(item) self:selectEgo(item, self.c_sfx_egos, 'suffix') end,
  }
  self.c_nitems = Numberbox.new {
    title = 'Number: ',
    number = 1,
    min = 1,
    max = 100,
    chars = 3,
    fct = function(text) end,
    on_change = function(_) self:updateButton() end,
  }
  self.c_no_id = Checkbox.new {
    title = 'No ID',
    default = false,
    on_change = function(s) if s then radio('id', 'c_no_id') end end
  }
  self.c_id = Checkbox.new {
    title = 'ID',
    default = true,
    on_change = function(s) if s then radio('id', 'c_id') end end
  }
  self.c_full_id = Checkbox.new {
    title = 'Full ID',
    default = false,
    on_change = function(s) if s then radio('id', 'c_full_id') end end
  }
  self.c_good = Checkbox.new {
    title = 'Force Good',
    default = false,
    on_change = function(s)
      if s then
	radio('power', 'c_good')
	self:updateButton()
      end
    end
  }
  self.c_great = Checkbox.new {
    title = 'Force Great',
    default = false,
    on_change = function(s)
      if s then
	radio('power', 'c_great')
	self:updateButton()
      end
    end
  }
  self.c_power = Checkbox.new {
    title = 'Force Power',
    default = false,
    on_change = function(s)
      if s then
	radio('power', 'c_power')
	self:updateButton()
      end
    end
  }
  self.c_n_power = Numberbox.new {
    title = ': ',
    number = 0,
    min = -2,
    max = 2,
    chars = 2,
    fct = function(text) end,
    on_change = function(_) self:updateButton() end,
  }
  self.c_randart = Checkbox.new {
    title = 'Force Randart',
    default = false,
    on_change = function(s) self:updateButton() end
  }
  self.c_mon = Checkbox.new {
    title = 'Force Mon ID',
    default = false,
    on_change = function(s) self:updateButton() end
  }
  self.c_mon_id = Textbox.new {
    title = ': ',
    text = '',
    chars = 16,
    max_chars = 48,
    fct = function () end,
  }
  self.c_create = Button.new {
    text = 'Create',
    fct = function() self:createItem() end,
  }
  self.c_cancel = Button.new {
    text = 'Cancel',
    fct = function() self.key:triggerVirtual('EXIT') end,
  }

  Dialog.init(self, 'Create Object', 800, 600)

  self:loadUI {
    { left=0, top=0, ui=self.c_groups },
    { right=0, top=0, ui=self.c_items },
    { left=0, top=self.c_groups, ui=self.c_nitems },
    { left=0, top=self.c_nitems, ui=self.c_no_id },
    { left=self.c_no_id.w+5, top=self.c_nitems, ui=self.c_id },
    { left=self.c_no_id.w+self.c_id.w+10, top=self.c_nitems, ui=self.c_full_id },
    { left=0, top=self.c_no_id, ui=self.c_good },
    { left=0, top=self.c_good, ui=self.c_great },
    { left=0, top=self.c_great, ui=self.c_power },
    { left=self.c_power, top=self.c_great, ui=self.c_n_power },
    { left=0, top=self.c_power, ui=self.c_randart },
    { left=0, top=self.c_randart, ui=self.c_mon },
    { left=self.c_mon, top=self.c_randart, ui=self.c_mon_id },
    { right=0, top=self.c_items.h, ui=self.c_sfx_egos },
    { right=self.c_sfx_egos, top=self.c_items.h, ui=self.c_pfx_egos },
    { right=0, bottom=0, ui=self.c_cancel },
    { right=self.c_cancel, bottom=0, ui=self.c_create },
  }
  self:setupUI(true, true)
  self:updateButton()

  self.key:addBinds {
    EXIT = function() game:unregisterDialog(self) end,
  }
end

function _M:on_register()
  game:onTickEnd(function() self.key:unicodeInput(true) end)
end

function _M:readyToCreate()
  if not self.sel_item or not self.c_nitems.number then return false end
  if self.c_power.checked and not self.c_n_power.number then return false end
  return true
end

function _M:updateButton()
  self:toggleDisplay(self.c_create, self:readyToCreate())
end

function _M:setDisplayText(item, which)
  local sel = which
  if type(sel) == 'string' then
    sel = (item.id and item.id == self.ego_names[which]) or
	  (not item.id and not self.ego_names[which])
  end
  item.display = sel and '#{bold}#'..item.name..'#{normal}#' or item.name
end

function _M:setupForGroup(item)
  if not item then return end
  local f = item.filter

  if self.sel_grp then
    self:setDisplayText(self.sel_grp, false)
    self.c_groups:drawItem(self.sel_grp)
  end
  self.sel_grp = item
  if self.sel_grp then
    self:setDisplayText(self.sel_grp, true)
    self.c_groups:drawItem(self.sel_grp)
  end

  local list = {}
  for _, o in ipairs(game.zone.object_list) do
    if o.name and o.type == f.type and o.subtype == f.subtype then
      list[#list+1] = { name = o:getName{force_qty=1, force_id=true}, tmpl = o }
    end
  end
  for _, o in ipairs(game.zone.artifacts_list) do
    if o.name and o.type == f.type and o.subtype == f.subtype then
      list[#list+1] = { name = o:getName{force_qty=1, force_id=true}, tmpl = o }
    end
  end
  for _, item in ipairs(list) do self:setDisplayText(item, false) end

  self.c_items.list = list
  self.c_items:generate()
  self:selectItem(nil)
  self:setFocus(self.c_items)
end

function _M:selectItem(item)
  if self.sel_item then
    self:setDisplayText(self.sel_item, false)
    self.c_items:drawItem(self.sel_item)
  end
  self.sel_item = item
  if self.sel_item then
    self:setDisplayText(self.sel_item, true)
    self.c_items:drawItem(self.sel_item)
  end
  self:updateButton()
  self:setupEgoLists()
end

function _M:setupEgoLists()
  if not self.sel_item or not self.sel_item.tmpl then
    self.c_pfx_egos.list = {}
    self.c_pfx_egos:generate()
    self.c_sfx_egos.list = {}
    self.c_sfx_egos:generate()
    self.ego_names = {}
    return
  end

  local Ego = require 'mod.class.Ego'

  local pfx_egos = { { name = 'No prefix ego' } }
  local sfx_egos = { { name = 'No suffix ego' } }
  for _, ego in ipairs(Ego:allowedEgosFor(self.sel_item.tmpl)) do
    local dst = ego.prefix and pfx_egos or ego.suffix and sfx_egos
    local item = { name = ego.name, id = ego.define_as }
    dst[item.id] = item
    table.insert(dst, item)
  end
  self.ego_names = {}
  for _, item in ipairs(pfx_egos) do self:setDisplayText(item, 'prefix') end
  for _, item in ipairs(sfx_egos) do self:setDisplayText(item, 'suffix') end

  self.c_pfx_egos.list = pfx_egos
  self.c_pfx_egos:generate()
  self.c_sfx_egos.list = sfx_egos
  self.c_sfx_egos:generate()
end

function _M:selectEgo(item, c_egos, which)
  local egos = c_egos.list
  local prev_sel = self.ego_names[which] and egos[self.ego_names[which]] or egos[1]
  self.ego_names[which] = item.id
  self:setDisplayText(prev_sel, which)
  c_egos:drawItem(prev_sel)
  self:setDisplayText(item, which)
  c_egos:drawItem(item)
end

function _M:createItem()
  if not self:readyToCreate() then return end
  game:unregisterDialog(self)

  local RF = (require 'mod.class.Object').resolve_flags
  if self.ego_names.prefix or self.ego_names.suffix then
    RF.force_ego = {}
    for _, id in pairs(self.ego_names) do table.insert(RF.force_ego, id) end
  end
  if self.c_good.checked then RF.force_good = true end
  if self.c_great.checked then RF.force_great = true end
  if self.c_power.checked and self.c_n_power.number then
    RF.force_level = self.c_n_power.number
  end
  if self.c_mon.checked and self.c_mon_id.text then
    RF.force_mon_id = self.c_mon_id.text
  end

  local qty = self.c_nitems.number
  local o = game.zone:finishEntity(game.level, 'object', self.sel_item.tmpl)
  if not o then
    game.log('??? Failed to create object')
    return
  end
  if not self.sel_item.tmpl.unique then
    o:setNumber(qty)
  end
  if self.c_full_id.checked then
    o:identify('full', game.player)
  elseif self.c_id.checked then
    o:identify(true, game.player)
  end
  if self.c_randart.checked and obj_util.randartable_kind(o) then
    obj_util.convert_to_randart(o, true)
  end
  o.found = { type = 'debug_dialog' }
  game.zone:addEntity(game.level, o, 'object', game.player.x, game.player.y)
end

function _M:generateGroupList()
  local list = {}
  local hits = {}

  for _, o in ipairs(game.zone.object_list) do
    if o.type and o.subtype then
      local key = o.type..'/'..o.subtype
      if not hits[key] then
	list[#list+1] = { name=key, filter={ type=o.type, subtype=o.subtype } }
	hits[key] = true
      end
    end
  end
  for _, item in ipairs(list) do self:setDisplayText(item, false) end

  self.groups = list
end
