-- $Id: CardLayout.lua 3574 2019-02-07 04:30:17Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2019 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

require 'engine.class'
local Base = require 'engine.ui.Base'
local Focusable = require "engine.ui.Focusable"
local SubGroup = require 'mod.dialogs.ui.SubGroup'

module(..., package.seeall, class.inherit(Base, Focusable))

function _M:init(grp_defs)
  Base.init(self, {}, true)
  self.cards = {}
  self.w, self.h = 0, 0

  local empty = {}
  for key, def in pairs(grp_defs) do
    if not next(def) then
      empty[key] = true
    else
      self.cards[key] = SubGroup.new(1, 1)
      self.cards[key]:loadUI(def)
      self.cards[key]:setupUI(true, true)
      self.w = math.max(self.w, self.cards[key].w)
      self.h = math.max(self.h, self.cards[key].h)
    end
  end
  for _, card in pairs(self.cards) do card:resize(self.w, self.h) end
  for key, _ in pairs(empty) do
    self.cards[key] = SubGroup.new(self.w, self.h)
  end
end

function _M:resize(w, h, nogen)
  self.w, self.h = math.floor(w), math.floor(h)
  self.ix, self.iy = 0, 0
  self.iw, self.ih = w, h

  for _, card in pairs(self.cards) do card:resize(w, h, true) end
  if not nogen then self:generate() end
end

function _M:generate()
  for _, card in pairs(self.cards) do card:generate() end
end

function _M:set_initial_card(key)
  if not self.cur_card then self.cur_card = key end
end

function _M:current_card()
  return self.cur_card and self.cards[self.cur_card] or nil
end

function _M:show_card(key, parent)
  if key ~= self.cur_card then
    parent:replaceUI(self.cards[self.cur_card], self.cards[key])
    self.cur_card = key
  end
end
