-- $Id: RadioButton.lua 3517 2018-12-15 04:32:06Z dsb $
-- ToME - Tales of Middle-Earth
-- Copyright © 2018 Scott Bigham
--
-- This program is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- This program is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with this program.  If not, see <http://www.gnu.org/licenses/>.
--
-- Scott Bigham "Zizzo"
-- dsb-tome@killerbunnies.org

-- A Checkbox modified for use as a radio button.  Radio buttons will be
-- grouped by their (required) 'radio_group' field; checking one will
-- automatically uncheck all the rest.  In addition, you can specify a
-- group callback via RadioButton:setGroupSelect(), which will be called
-- when one of the buttons in the group is checked with the checked button
-- as a parameter; your callback can use the optional 'radio_id' field to
-- identify which button it is receiving.
require 'engine.class'
local Checkbox = require 'engine.ui.Checkbox'

module(..., package.seeall, class.inherit(Checkbox))

_M.groups = {}
_M.group_cbs = {}

function _M:init(t)
  self.radio_group = assert(t.radio_group, 'no radio button radio_group')
  self.radio_id = t.radio_id

  Checkbox.init(self, t)

  local grp = table.getTable(self.groups, self.radio_group)
  grp[#grp+1] = self
end

function _M:generate()
  Checkbox.generate(self)
  -- We've ensured that ui/radio{,-ok}.png have the same size as
  -- ui/checkbox{,-ok}.png, so we don't have to redo self.{w,h}.
  self.check = self:getUITexture('ui/radio.png')
  self.tick = self:getUITexture('ui/radio-ok.png')
end

function _M:select()
  Checkbox.select(self)
  if self.checked then
    for _, btn in ipairs(self.groups[self.radio_group]) do
      if btn ~= self and btn.checked then
	btn.checked = false
	if btn.on_change then btn.on_change(false) end
      end
    end
    if self.group_cbs[self.radio_group] then
      self.group_cbs[self.radio_group](self)
    end
  end
end

-- Static method
function _M:setGroupSelect(grp, f)
  self.group_cbs[grp] = f
end
